import React, { useState, useRef } from 'react'
import { useTranslation } from 'react-i18next'

import { Box, Button } from '@mui/material'
import Page from 'components/page'

import { useHistory } from 'react-router-dom'
import format from 'date-fns/format'
import { RegisterFormData } from './formData'
import StepSummary from './stepSummary'
import StepPersonForm from './stepPersonForm'
import { PersonFormMethods } from './personFormMethods'
import SubmitState from './submitState'
import SponsorGuestButtons from '../../components/sponsorGuestButtons'
import { submitJsonOpts } from '../../../utils'
import StepSubmitSuccess from './stepSubmitSuccess'

enum Steps {
  RegisterStep = 0,
  SummaryStep = 1,
  SuccessStep = 2,
}

/**
 *
 * This component controls the invite process where the sponsor
 * enters the initial information about a guest.
 *
 */
export default function StepRegistration() {
  const { t } = useTranslation(['common'])
  const [formData, setFormData] = useState<RegisterFormData>({
    first_name: undefined,
    last_name: undefined,
    role_type: undefined,
    role_start: undefined,
    role_end: undefined,
    comment: undefined,
    ou_id: undefined,
    email: undefined,
  })
  const history = useHistory()

  const [activeStep, setActiveStep] = useState(0)
  const personFormRef = useRef<PersonFormMethods>(null)
  const [submitState, setSubmitState] = useState(SubmitState.NotSubmitted)

  const handleNext = () => {
    if (activeStep === 0) {
      if (personFormRef.current) {
        personFormRef.current.doSubmit()
      }
    } else {
      setActiveStep((prevActiveStep) => prevActiveStep + 1)
    }
  }

  const registerGuest = () => {
    const payload = {
      first_name: formData.first_name,
      last_name: formData.last_name,
      email: formData.email,
      role: {
        type: formData.role_type,
        start_date:
          formData.role_start === null
            ? null
            : format(formData.role_start as Date, 'yyyy-MM-dd'),
        end_date:
          formData.role_end === null
            ? null
            : format(formData.role_end as Date, 'yyyy-MM-dd'),
        comments: formData.comment,
        orgunit_id: formData.ou_id,
      },
    }

    console.log('submitting', JSON.stringify(payload))
    fetch('/api/ui/v1/invite/', submitJsonOpts('POST', payload))
      .then((res) => {
        if (!res.ok) {
          setSubmitState(SubmitState.SubmitFailure)
          return null
        }
        return res.text()
      })
      .then((result) => {
        if (result !== null) {
          console.log('result', result)
          setSubmitState(SubmitState.SubmitSuccess)
          setActiveStep(Steps.SuccessStep)
        }
      })
      .catch((error) => {
        console.log('error', error)
        setSubmitState(SubmitState.SubmitFailure)
      })
  }

  const handleBack = () => {
    setActiveStep((prevActiveStep) => prevActiveStep - 1)
  }

  const handleForwardFromRegister = (
    updateFormData: RegisterFormData
  ): void => {
    setFormData(updateFormData)
    setActiveStep((prevActiveStep) => prevActiveStep + 1)
  }

  const handleCancel = () => {
    history.push('/')
  }

  return (
    <Page>
      <SponsorGuestButtons registerNewGuestActive />
      {/* Current page in wizard */}
      <Box sx={{ width: '100%' }}>
        {activeStep === Steps.RegisterStep && (
          <StepPersonForm
            nextHandler={handleForwardFromRegister}
            formData={formData}
            ref={personFormRef}
          />
        )}
        {activeStep === Steps.SummaryStep && (
          <StepSummary formData={formData} />
        )}
      </Box>

      <Box
        sx={{
          display: 'flex',
          flexDirection: 'row',
          pt: 2,
          color: 'primary.main',
          paddingBottom: '1rem',
        }}
      >
        {activeStep === Steps.RegisterStep && (
          <Button
            data-testid="button-next"
            sx={{ color: 'theme.palette.secondary', mr: 1 }}
            onClick={handleNext}
          >
            {t('button.next')}
          </Button>
        )}

        {activeStep === Steps.SummaryStep && (
          <>
            <Button
              onClick={handleBack}
              disabled={submitState === SubmitState.SubmitSuccess}
              sx={{ mr: 1 }}
            >
              {t('button.back')}
            </Button>

            <Button
              onClick={registerGuest}
              disabled={submitState === SubmitState.SubmitSuccess}
              sx={{ mr: 1 }}
            >
              {t('button.save')}
            </Button>
          </>
        )}

        {activeStep !== Steps.SuccessStep && (
          <Button
            onClick={handleCancel}
            disabled={submitState === SubmitState.SubmitSuccess}
          >
            {t('button.cancel')}
          </Button>
        )}
      </Box>

      {activeStep === Steps.SuccessStep && <StepSubmitSuccess />}

      {/* TODO For now just showing a heading to give the user some feedback */}
      {submitState === SubmitState.SubmitFailure && (
        <Box>
          <h2>Submit failure</h2>
        </Box>
      )}
    </Page>
  )
}
