from typing import (
    Sequence,
    Type,
)

from rest_framework import permissions
from rest_framework.authentication import BaseAuthentication, SessionAuthentication
from rest_framework.permissions import AllowAny, BasePermission
from rest_framework.status import HTTP_403_FORBIDDEN
from rest_framework.views import APIView
from rest_framework.response import Response
from greg.models import Identity, InvitationLink

from gregui.models import GregUserProfile


class UserInfoView(APIView):
    """
    User info view.

    Return info about the logged inn user.
    Quick draft, we might want to expand this later.
    """

    authentication_classes: Sequence[Type[BaseAuthentication]] = [SessionAuthentication]
    permission_classes: Sequence[Type[BasePermission]] = [AllowAny]

    def get(self, request, format=None):
        """
        Get info about the visiting user

        Works for users logged in using Feide, and those relying solely on an
        invitation id.

        TODO: Can this be modified into a permission class to reduce clutter?
        """
        user = request.user
        invite_id = request.session.get("invite_id")

        # Authenticated user, allow access
        if user.is_authenticated:
            user_profile = GregUserProfile.objects.get(user=user)
            sponsor_id = None
            person_id = None
            if user_profile.sponsor:
                sponsor_id = user_profile.sponsor.id
            if user_profile.person:
                person_id = user_profile.person.id
            content = {
                "feide_id": user_profile.userid_feide,
                "sponsor_id": sponsor_id,
                "person_id": person_id,
            }
            person = user_profile.person
            if person:
                content.update(
                    {
                        "first_name": person.first_name,
                        "last_name": person.last_name,
                        "email": person.private_email and person.private_email.value,
                        "mobile_phone": person.private_mobile
                        and person.private_mobile.value,
                    }
                )
                roles = person.roles
                if roles:
                    content.update(
                        {
                            "roles": [
                                {
                                    "ou_name_nb": role.orgunit_id.name_nb,
                                    "ou_name_en": role.orgunit_id.name_en,
                                    "role_name_nb": role.type.name_nb,
                                    "role_name_en": role.type.name_en,
                                    "start": role.start_date,
                                    "end": role.end_date,
                                    "comments": role.comments,
                                    "sponsor": {
                                        "first_name": role.sponsor_id.first_name,
                                        "last_name": role.sponsor_id.last_name,
                                    },
                                }
                                for role in roles.all()
                            ],
                        }
                    )
            return Response(content)

        # Invitation cookie, allow access
        elif invite_id:
            link = InvitationLink.objects.get(uuid=invite_id)
            invitation = link.invitation
            person = invitation.role.person
            passports = person.identities.filter(
                type=Identity.IdentityType.PASSPORT_NUMBER
            ).first()

            content = {
                "feide_id": None,
                "first_name": person.first_name,
                "last_name": person.last_name,
                "email": person.private_email and person.private_email.value,
                "mobile_phone": person.private_mobile and person.private_mobile.value,
                "fnr": person.fnr and person.fnr.value,
                "passport": passports and passports.value,
                "roles": [
                    {
                        "ou_name_nb": role.orgunit_id.name_nb,
                        "ou_name_en": role.orgunit_id.name_en,
                        "role_name_nb": role.type.name_nb,
                        "role_name_en": role.type.name_en,
                        "start": role.start_date,
                        "end": role.end_date,
                        "comments": role.comments,
                        "sponsor": {
                            "first_name": role.sponsor_id.first_name,
                            "last_name": role.sponsor_id.last_name,
                        },
                    }
                    for role in person.roles.all()
                ],
            }

            return Response(content)

        # Neither, deny access
        else:
            return Response(status=HTTP_403_FORBIDDEN)
